==> autogen-5.3.8.tar.gz: autogen-5.3.8/autoopts/restore.c <==

/*
 *  restore.c  $Id: restore.c,v 3.2 2002/05/11 20:23:52 bkorb Exp $
 *
 *  This module's routines will save the current option state to memory
 *  and restore it.  If saved prior to the initial optionProcess call,
 *  then the initial state will be restored.
 */

/*
 *  Automated Options copyright 1992-2002 Bruce Korb
 *
 *  Automated Options is free software.
 *  You may redistribute it and/or modify it under the terms of the
 *  GNU General Public License, as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  Automated Options is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Automated Options.  See the file "COPYING".  If not,
 *  write to:  The Free Software Foundation, Inc.,
 *             59 Temple Place - Suite 330,
 *             Boston,  MA  02111-1307, USA.
 *
 * As a special exception, Bruce Korb gives permission for additional
 * uses of the text contained in his release of AutoOpts.
 *
 * The exception is that, if you link the AutoOpts library with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the AutoOpts library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by Bruce Korb under
 * the name AutoOpts.  If you copy code from other sources under the
 * General Public License into a copy of AutoOpts, as the General Public
 * License permits, the exception does not apply to the code that you add
 * in this way.  To avoid misleading anyone as to the status of such
 * modified files, you must delete this exception notice from them.
 *
 * If you write modifications of your own for AutoOpts, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 */

#include "autoopts.h"

/*=export_func optionSaveState
 *
 * what:  saves the option state to memory
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:   This routine will allocate enough memory to save the current
 *        option processing state.  If this routine has been called before,
 *        that memory will be reused.  You may only save one copy of the
 *        option state.  This routine may be called before optionProcess(3).
 *
 * err:   If it fails to allocate the memory,
 *        it will print a message to stderr and exit.
 *        Otherwise, it will always succeed.
=*/
void
optionSaveState( pOpts )
    tOptions* pOpts;
{
    if (pOpts->pSavedState == NULL) {
        size_t sz = sizeof( *pOpts ) + (pOpts->optCt * sizeof( tOptDesc ));
        pOpts->pSavedState = AGALOC( sz );
        if (pOpts->pSavedState == NULL) {
            tCC* pzName = pOpts->pzProgName;
            if (pzName == NULL) {
                pzName = pOpts->pzPROGNAME;
                if (pzName == NULL)
                    pzName = "";
            }
            fprintf( stderr, "%s(optionSaveState): error: cannot allocate "
                     "%d bytes\n", pzName, sz );
            exit( EXIT_FAILURE );
        }
    }

    {
        tOptions* p = pOpts->pSavedState;
        memcpy( p, pOpts, sizeof( *p ));
        memcpy( p + 1, pOpts->pOptDesc, p->optCt * sizeof( tOptDesc ));
    }
}


/*=export_func optionRestore
 *
 * what:  restore option state from memory copy
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:  Copy back the option state from saved memory.
 *       The allocated memory is left intact, so this routine can be
 *       called repeatedly without having to call optionSaveState again.
 *
 * err:  If you have not called @code{optionSaveState} before, a diagnostic is
 *       printed to @code{stderr} and exit is called.
=*/
void
optionRestore( pOpts )
    tOptions* pOpts;
{
    tOptions* p = (tOptions*)pOpts->pSavedState;

    if (p == NULL) {
        tCC* pzName = pOpts->pzProgName;
        if (pzName == NULL) {
            pzName = pOpts->pzPROGNAME;
            if (pzName == NULL)
                pzName = "";
        }
        fprintf( stderr, "%s(optionRestore): error: no saved option state\n",
                 pzName );
        exit( EXIT_FAILURE );
    }
    memcpy( pOpts, p, sizeof( *p ));
    memcpy( pOpts->pOptDesc, p+1, p->optCt * sizeof( tOptDesc ));
}

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/*=export_func optionFree
 *
 * what:  free allocated option processing memory
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:   AutoOpts sometimes allocates memory and puts pointers to it in the
 *        option state structures.  This routine deallocates all such memory.
 *
 * err:   As long as memory has not been corrupted,
 *        this routine is always successful.      
=*/
void
optionFree( pOpts )
    tOptions* pOpts;
{
    if (pOpts->pSavedState != NULL) {
        AGFREE( pOpts->pSavedState );
        pOpts->pSavedState = NULL;
    }
    {
        tOptDesc* p = pOpts->pOptDesc;
        int ct = pOpts->optCt;
        do  {
            if (p->optCookie != NULL) {
                AGFREE( p->optCookie );
                p->fOptState &= OPTST_PERSISTENT;
                if ((p->fOptState & OPTST_INITENABLED) == 0)
                    p->fOptState |= OPTST_DISABLED;
            }
        } while (p++, --ct > 0);
    }
}
/*
 * Local Variables:
 * c-file-style: "stroustrup"
 * indent-tabs-mode: nil
 * End:
 * restore.c ends here */

==> autogen-5.5.2.tar.gz: autogen-5.5.2/autoopts/restore.c <==

/*
 *  restore.c  $Id: restore.c,v 2.4 2003/02/15 22:58:59 bkorb Exp $
 *
 *  This module's routines will save the current option state to memory
 *  and restore it.  If saved prior to the initial optionProcess call,
 *  then the initial state will be restored.
 */

/*
 *  Automated Options copyright 1992-2003 Bruce Korb
 *
 *  Automated Options is free software.
 *  You may redistribute it and/or modify it under the terms of the
 *  GNU General Public License, as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  Automated Options is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Automated Options.  See the file "COPYING".  If not,
 *  write to:  The Free Software Foundation, Inc.,
 *             59 Temple Place - Suite 330,
 *             Boston,  MA  02111-1307, USA.
 *
 * As a special exception, Bruce Korb gives permission for additional
 * uses of the text contained in his release of AutoOpts.
 *
 * The exception is that, if you link the AutoOpts library with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the AutoOpts library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by Bruce Korb under
 * the name AutoOpts.  If you copy code from other sources under the
 * General Public License into a copy of AutoOpts, as the General Public
 * License permits, the exception does not apply to the code that you add
 * in this way.  To avoid misleading anyone as to the status of such
 * modified files, you must delete this exception notice from them.
 *
 * If you write modifications of your own for AutoOpts, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 */

#include "autoopts.h"

/*=export_func optionSaveState
 *
 * what:  saves the option state to memory
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:   This routine will allocate enough memory to save the current
 *        option processing state.  If this routine has been called before,
 *        that memory will be reused.  You may only save one copy of the
 *        option state.  This routine may be called before optionProcess(3).
 *
 * err:   If it fails to allocate the memory,
 *        it will print a message to stderr and exit.
 *        Otherwise, it will always succeed.
=*/
void
optionSaveState( pOpts )
    tOptions* pOpts;
{
    if (pOpts->pSavedState == NULL) {
        size_t sz = sizeof( *pOpts ) + (pOpts->optCt * sizeof( tOptDesc ));
        pOpts->pSavedState = AGALOC( sz, "saved option state" );
        if (pOpts->pSavedState == NULL) {
            tCC* pzName = pOpts->pzProgName;
            if (pzName == NULL) {
                pzName = pOpts->pzPROGNAME;
                if (pzName == NULL)
                    pzName = "";
            }
            fprintf( stderr, "%s(optionSaveState): error: cannot allocate "
                     "%d bytes\n", pzName, sz );
            exit( EXIT_FAILURE );
        }
    }

    {
        tOptions* p = pOpts->pSavedState;
        memcpy( p, pOpts, sizeof( *p ));
        memcpy( p + 1, pOpts->pOptDesc, p->optCt * sizeof( tOptDesc ));
    }
}


/*=export_func optionRestore
 *
 * what:  restore option state from memory copy
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:  Copy back the option state from saved memory.
 *       The allocated memory is left intact, so this routine can be
 *       called repeatedly without having to call optionSaveState again.
 *
 * err:  If you have not called @code{optionSaveState} before, a diagnostic is
 *       printed to @code{stderr} and exit is called.
=*/
void
optionRestore( pOpts )
    tOptions* pOpts;
{
    tOptions* p = (tOptions*)pOpts->pSavedState;

    if (p == NULL) {
        tCC* pzName = pOpts->pzProgName;
        if (pzName == NULL) {
            pzName = pOpts->pzPROGNAME;
            if (pzName == NULL)
                pzName = "";
        }
        fprintf( stderr, "%s(optionRestore): error: no saved option state\n",
                 pzName );
        exit( EXIT_FAILURE );
    }
    memcpy( pOpts, p, sizeof( *p ));
    memcpy( pOpts->pOptDesc, p+1, p->optCt * sizeof( tOptDesc ));
}

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/*=export_func optionFree
 *
 * what:  free allocated option processing memory
 * arg:   tOptions*, pOpts, program options descriptor
 *
 * doc:   AutoOpts sometimes allocates memory and puts pointers to it in the
 *        option state structures.  This routine deallocates all such memory.
 *
 * err:   As long as memory has not been corrupted,
 *        this routine is always successful.      
=*/
void
optionFree( pOpts )
    tOptions* pOpts;
{
    if (pOpts->pSavedState != NULL) {
        AGFREE( pOpts->pSavedState );
        pOpts->pSavedState = NULL;
    }
    {
        tOptDesc* p = pOpts->pOptDesc;
        int ct = pOpts->optCt;
        do  {
            if (p->optCookie != NULL) {
                AGFREE( p->optCookie );
                p->fOptState &= OPTST_PERSISTENT;
                if ((p->fOptState & OPTST_INITENABLED) == 0)
                    p->fOptState |= OPTST_DISABLED;
            }
        } while (p++, --ct > 0);
    }
}
/*
 * Local Variables:
 * c-file-style: "stroustrup"
 * indent-tabs-mode: nil
 * End:
 * restore.c ends here */
